@@ -170,10 +170,6 @@ BuildPrefsSaveFrame(void)
   // save to stdout:
   gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(main_window, "save_to_stdout"),camera->prefs.save_to_stdout>0);
 
-  // ram buffer
-  gtk_toggle_button_set_active((GtkToggleButton*)lookup_widget(main_window, "use_ram_buffer"),camera->prefs.use_ram_buffer);
-  gtk_spin_button_set_value((GtkSpinButton*)lookup_widget(main_window,"ram_buffer_size"), camera->prefs.ram_buffer_size);
-
 }
 
 void
