@@ -245,6 +245,7 @@ BuildCameraMenu(void)
   //eprint("uioyiuyb\n");
   // sets the active menu item:
   gtk_option_menu_set_history (GTK_OPTION_MENU (camera_id) , current_camera_id);
+  gtk_widget_set_sensitive (GTK_WIDGET (camera_id) , i>1);
 
   //eprint("exit\n");
 }
@@ -262,6 +263,7 @@ BuildFormat7ModeMenu(void)
   GtkWidget* mode_num;
   GtkWidget* mode_num_menu;
   GtkWidget* glade_menuitem;
+  char string[256];
 
   gtk_widget_destroy(GTK_WIDGET (lookup_widget(main_window,"format7_mode"))); // remove previous menu
 
@@ -279,7 +281,11 @@ BuildFormat7ModeMenu(void)
     if (camera->format7_info.modeset.mode[f-DC1394_VIDEO_MODE_FORMAT7_MIN].present>0) {
       index[i]=k;
       k++;
-      glade_menuitem = gtk_menu_item_new_with_label (_(format7_mode_list[i]));
+      sprintf(string, "Mode %d: %dx%d",f-DC1394_VIDEO_MODE_FORMAT7_MIN,
+              camera->format7_info.modeset.mode[f-DC1394_VIDEO_MODE_FORMAT7_MIN].max_size_x,
+              camera->format7_info.modeset.mode[f-DC1394_VIDEO_MODE_FORMAT7_MIN].max_size_y);
+      glade_menuitem = gtk_menu_item_new_with_label (_(string));
+
       gtk_widget_show (glade_menuitem);
       gtk_menu_append (GTK_MENU (mode_num_menu), glade_menuitem);
       g_signal_connect ((gpointer) glade_menuitem, "activate",
@@ -424,7 +430,7 @@ BuildFormatMenu(void)
   GtkWidget* mode_num_menu;
   GtkWidget* glade_menuitem;
   dc1394video_modes_t modes;
-
+  char string[256];
   //eprint("building format menu\n");
 
   dc1394video_mode_t video_mode;
@@ -450,14 +456,22 @@ BuildFormatMenu(void)
     return;
   }
 
+  int roi=0;
   for (i=0;i<modes.num;i++) {
-    glade_menuitem = gtk_menu_item_new_with_label (_(video_mode_list[modes.modes[i]-DC1394_VIDEO_MODE_MIN]));
-
-    gtk_widget_show (glade_menuitem);
-    gtk_menu_append (GTK_MENU (mode_num_menu), glade_menuitem);
-    g_signal_connect ((gpointer) glade_menuitem, "activate",
-		      G_CALLBACK (ChangeModeAndFormat),
-		      (gpointer)(unsigned long)modes.modes[i]);
+      if (!dc1394_is_video_mode_scalable(modes.modes[i]))
+          glade_menuitem = gtk_menu_item_new_with_label (_(video_mode_list[modes.modes[i]-DC1394_VIDEO_MODE_MIN]));
+      else {
+          sprintf(string, "%s %d: %dx%d",video_mode_list[modes.modes[i]-DC1394_VIDEO_MODE_MIN], roi,
+                  camera->format7_info.modeset.mode[modes.modes[i]-DC1394_VIDEO_MODE_FORMAT7_MIN].max_size_x,
+                  camera->format7_info.modeset.mode[modes.modes[i]-DC1394_VIDEO_MODE_FORMAT7_MIN].max_size_y);
+          glade_menuitem = gtk_menu_item_new_with_label (_(string));
+          roi++;
+      }
+      gtk_widget_show (glade_menuitem);
+      gtk_menu_append (GTK_MENU (mode_num_menu), glade_menuitem);
+      g_signal_connect ((gpointer) glade_menuitem, "activate",
+                        G_CALLBACK (ChangeModeAndFormat),
+                        (gpointer)(unsigned long)modes.modes[i]);
   }
   for (i=0;i<modes.num;i++) {
     if (video_mode==modes.modes[i])
@@ -948,13 +962,22 @@ BuildSaveFormatMenu(void)
 		      (int*)SAVE_FORMAT_PPM);
   
 #ifdef HAVE_FFMPEG
-  // only available with ffmpeg since we don;t have imlib anymore
+  // only available with ffmpeg since we don't have imlib anymore
   glade_menuitem = gtk_menu_item_new_with_label (_("jpeg"));
   gtk_widget_show (glade_menuitem);
   gtk_menu_append (GTK_MENU (new_menu), glade_menuitem);
   g_signal_connect ((gpointer) glade_menuitem, "activate",
 		      G_CALLBACK (on_save_format_menu_activate),
 		      (int*)SAVE_FORMAT_JPEG);
+#endif
+#ifdef HAVE_LIBTIFF
+  // only available with libtiff
+  glade_menuitem = gtk_menu_item_new_with_label (_("tiff"));
+  gtk_widget_show (glade_menuitem);
+  gtk_menu_append (GTK_MENU (new_menu), glade_menuitem);
+  g_signal_connect ((gpointer) glade_menuitem, "activate",
+		      G_CALLBACK (on_save_format_menu_activate),
+		      (int*)SAVE_FORMAT_TIFF);
 #endif  
 
   // DUMMY
