@@ -22,6 +22,9 @@
 void
 BuildTriggerModeMenu(void);
 
+void
+BuildTriggerSourceMenu(void);
+
 void
 BuildMemoryChannelMenu(void);
 
@@ -34,6 +37,9 @@ BuildFormat7ColorMenu(void);
 void
 BuildFormat7ModeMenu(void);
 
+void
+BuildIsoSpeedMenu(void);
+
 void
 BuildFpsMenu(void);
 
@@ -55,4 +61,13 @@ BuildOverlayTypeMenu(void);
 void
 BuildOverlayPatternMenu(void);
 
+void
+BuildSaveFormatMenu(void);
+
+void
+BuildSaveAppendMenu(void);
+
+void
+BuildRegisterAccessOffsetMenu(void);
+
 #endif // __BUILDMENUS_H__
