@@ -20,15 +20,15 @@
 #define __BUILDRANGES_H__
 
 void
-BuildRange(int feature);
+BuildRange(int feature, int *pos);
 
 void
-BuildEmptyRange(int feature);
+BuildEmptyRange(int feature, int pos);
 
 void 
 BuildFormat7Ranges(void);
 
 void
-BuildFormat7BppRange(void);
+BuildFormat7PacketSizeRange(void);
 
 #endif // __BUILDRANGES_H__
