@@ -1072,106 +1072,8 @@ on_prefs_update_power_toggled          (GtkToggleButton *togglebutton,
   gnome_config_set_int("coriander/global/auto_update",preferences.auto_update);
   gnome_config_sync();
 }
-/*
-void
-on_prefs_save_seq_toggled              (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  savethread_info_t* info;
-  chain_t* service;
-  if (togglebutton->active) {
-    camera->prefs.save_mode=SAVE_MODE_SEQUENTIAL;
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(main_window,"use_ram_buffer")),0);
-    gnome_config_set_int("coriander/save/mode",camera->prefs.save_mode);
-    gnome_config_sync();
-    UpdatePrefsSaveFrame();
-    service=GetService(camera,SERVICE_SAVE);
-    if (service!=NULL) {
-      info=service->data;
-      info->mode=camera->prefs.save_mode;
-    }
-  }
-}
 
 
-void
-on_prefs_save_mode_toggled          (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  savethread_info_t* info;
-  chain_t* service;
-  if (togglebutton->active) {
-    camera->prefs.save_mode=SAVE_MODE_OVERWRITE;
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget(main_window,"use_ram_buffer")),0);
-    gnome_config_set_int("coriander/save/mode",camera->prefs.save_mode);
-    gnome_config_sync();
-    UpdatePrefsSaveFrame();
-    service=GetService(camera,SERVICE_SAVE);
-    if (service!=NULL) {
-      info=service->data;
-      info->mode=camera->prefs.save_mode;
-    }
-  }
-}
-
-
-void
-on_prefs_save_video_toggled            (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  savethread_info_t* info;
-  chain_t* service;
-  if (togglebutton->active) {
-    camera->prefs.save_mode=SAVE_MODE_VIDEO;
-    gnome_config_set_int("coriander/save/mode",camera->prefs.save_mode);
-    gnome_config_sync();
-    UpdatePrefsSaveFrame();
-    service=GetService(camera,SERVICE_SAVE);
-    if (service!=NULL) {
-      info=service->data;
-      info->mode=camera->prefs.save_mode;
-    }
-  }
-}
-
-void
-on_prefs_save_convert_toggled          (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  savethread_info_t* info;
-  chain_t* service;
-  if (togglebutton->active)
-    camera->prefs.save_convert=SAVE_CONVERT_ON;
-  gnome_config_set_int("coriander/save/convert",camera->prefs.save_convert);
-  gnome_config_sync();
-  UpdatePrefsSaveFrame();
-  service=GetService(camera,SERVICE_SAVE);
-  if (service!=NULL) {
-    info=service->data;
-    info->rawdump=camera->prefs.save_convert;
-  }
-}
-
-
-void
-on_prefs_save_noconvert_toggled        (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  savethread_info_t* info;
-  chain_t* service;
-  if (togglebutton->active)
-    camera->prefs.save_convert=SAVE_CONVERT_OFF;
-  gnome_config_set_int("coriander/save/convert",camera->prefs.save_convert);
-  gnome_config_sync();
-  UpdatePrefsSaveFrame();
-  service=GetService(camera,SERVICE_SAVE);
-  if (service!=NULL) {
-    info=service->data;
-    info->rawdump=camera->prefs.save_convert;
-  }
-}
-*/
-
 void
 on_prefs_ftp_seq_toggled               (GtkToggleButton *togglebutton,
                                         gpointer         user_data)
@@ -1432,46 +1334,6 @@ on_prefs_ftp_num_tag_toggled           (GtkToggleButton *togglebutton,
 }
 
 
-void
-on_ram_buffer_toggled                  (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
-{
-  camera->prefs.use_ram_buffer=togglebutton->active;
-  gnome_config_set_int("coriander/save/use_ram_buffer",camera->prefs.use_ram_buffer);
-  gnome_config_sync();
-  UpdatePrefsSaveFrame();
-}
-
-
-void
-on_ram_buffer_size_changed             (GtkEditable     *editable,
-                                        gpointer         user_data)
-{
-  camera->prefs.ram_buffer_size=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(lookup_widget(main_window,"ram_buffer_size")));
-  gnome_config_set_int("coriander/save/ram_buffer_size",camera->prefs.ram_buffer_size);
-  gnome_config_sync();
-  //fprintf(stderr,"Buffer size: %d\n",camera->prefs.ram_buffer_size);
-  UpdatePrefsSaveFrame();
-}
-
-
-void
-on_malloc_test_clicked                 (GtkButton       *button,
-                                        gpointer         user_data)
-{
-  unsigned char *temp;
-
-  // test if we can allocate enough memory
-  temp=(unsigned char*)malloc(camera->prefs.ram_buffer_size*1024*1024*sizeof(unsigned char));
-  
-  if (temp==NULL)
-    Warning("\tFailed to allocate memory");
-  else {
-    Warning("\tAllocation succeeded");
-    free(temp);
-  }
-}
-
 
 void
 on_dma_buffer_size_changed             (GtkEditable     *editable,
@@ -1648,6 +1510,11 @@ on_save_format_menu_activate             (GtkEditable     *editable,
   case SAVE_FORMAT_JPEG:
       sprintf(camera->prefs.save_filename_ext,"jpeg");
       break;
+#endif
+#ifdef HAVE_LIBTIFF
+  case SAVE_FORMAT_TIFF:
+      sprintf(camera->prefs.save_filename_ext,"tiff");
+      break;
 #endif
   case SAVE_FORMAT_RAW_VIDEO:
       sprintf(camera->prefs.save_filename_ext,"raw");
