@@ -24,7 +24,7 @@ DisplayStartThread(camera_t* cam)
   chain_t *display_service=NULL;
   displaythread_info_t *info=NULL;
 
-  display_service=GetService(camera,SERVICE_DISPLAY);
+  display_service=GetService(cam,SERVICE_DISPLAY);
 
   if (display_service==NULL) {// if no display service running...
     display_service=(chain_t*)malloc(sizeof(chain_t));
@@ -41,10 +41,9 @@ DisplayStartThread(camera_t* cam)
     pthread_mutex_unlock(&info->mutex_cancel);
     
     pthread_mutex_lock(&display_service->mutex_data);
-    info->period=cam->prefs.display_period;
     CommonChainSetup(cam, display_service,SERVICE_DISPLAY);
-    
     pthread_mutex_lock(&display_service->mutex_struct);
+
     InsertChain(cam, display_service);
     
     if (pthread_create(&display_service->thread, NULL, DisplayThread, (void*)display_service)) {
@@ -63,21 +62,6 @@ DisplayStartThread(camera_t* cam)
 }
 
 
-void*
-DisplayCleanupThread(void* arg)
-{
-  chain_t* display_service;
-  displaythread_info_t *info;
-
-  display_service=(chain_t*)arg;
-  info=(displaythread_info_t*)display_service->data;
-
-  pthread_mutex_unlock(&display_service->mutex_data);
-
-  return(NULL);
-}
-
-
 void*
 DisplayThread(void* arg)
 {
@@ -95,10 +79,12 @@ DisplayThread(void* arg)
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED,NULL);
   pthread_mutex_unlock(&display_service->mutex_data);
   skip_counter=0;
+  camera_t *cam=display_service->camera;
 
   // time inits:
-  display_service->prev_time = times(&display_service->tms_buf);
-  display_service->fps_frames=0;
+  display_service->prev_time = 0;
+  display_service->prev_period = 0;
+  display_service->drop_warning = 0;
   display_service->processed_frames=0;
 
   while (1) {
@@ -110,23 +96,21 @@ DisplayThread(void* arg)
     else {
       pthread_mutex_unlock(&info->mutex_cancel);
       pthread_mutex_lock(&display_service->mutex_data);
-      if(RollBuffers(display_service)) { // have buffers been rolled?
+      if(GetBufferFromPrevious(display_service)) { // have buffers been rolled?
 #ifdef HAVE_SDLLIB
 	// check params
 	DisplayThreadCheckParams(display_service);
 #endif
-	if (display_service->current_buffer->width!=-1) {
-	  if (skip_counter>=(info->period-1)) {
+	if (display_service->current_buffer->frame.size[0]!=-1) {
+	  if (skip_counter>=(cam->prefs.display_period-1)) {
 	    skip_counter=0;
 #ifdef HAVE_SDLLIB
 	    if (info->sdloverlay!=NULL) {
 	      if (SDL_LockYUVOverlay(info->sdloverlay) == 0) {
-		convert_to_yuv_for_SDL(display_service->current_buffer, info->sdloverlay, preferences.overlay_byte_order);
-		
+		convert_to_yuv_for_SDL(&display_service->current_buffer->frame, info->sdloverlay, preferences.overlay_byte_order);
 		// informative overlays
-		SDLDisplayArea(display_service);
 		SDLDisplayPattern(display_service);
-		
+		SDLDisplayArea(display_service);
 		SDL_UnlockYUVOverlay(info->sdloverlay);
 		SDL_DisplayYUVOverlay(info->sdloverlay, &info->sdlvideorect);
 		
@@ -134,34 +118,47 @@ DisplayThread(void* arg)
 	      }
 	    }
 #endif
-	    display_service->fps_frames++;
 	    display_service->processed_frames++;
 	  }
 	  else { //
-	    if (display_service->camera->prefs.display_redraw==DISPLAY_REDRAW_ON) {
+	    if (cam->prefs.display_redraw==DISPLAY_REDRAW_ON) {
 	      ConditionalTimeoutRedraw(display_service);
 	    }
 	    skip_counter++;
 	  }
-	  // FPS display:
-	  display_service->current_time=times(&display_service->tms_buf);
-
-	  tmp=(float)(display_service->current_time-display_service->prev_time)/sysconf(_SC_CLK_TCK);
-	  if (tmp==0)
+	  // FPS computation:
+	  tmp=((float)(display_service->current_buffer->frame.timestamp-display_service->prev_time))/1000000.0;
+	  if (display_service->prev_time==0) {
 	    display_service->fps=fabs(0.0);
-	  else
-	    display_service->fps=fabs((float)display_service->fps_frames/tmp);
+	  }
+	  else {
+	    if (tmp==0)
+	      display_service->fps=fabs(0.0);
+	    else
+	      display_service->fps=fabs(1/tmp);
+	  }
+	  if (display_service->prev_time!=0) {
+	    display_service->prev_period=tmp;
+	  }
+	  // produce a drop warning if the period difference is over 50%
+	  if (display_service->prev_period!=0) {
+	    if (fabs(display_service->prev_period-tmp)/(display_service->prev_period/2+tmp/2)>=.5)
+	      display_service->drop_warning++;
+	  }
+	  display_service->prev_time=display_service->current_buffer->frame.timestamp;
 	}
+
+	PublishBufferForNext(display_service);
 	pthread_mutex_unlock(&display_service->mutex_data);
       }
       else { //
-	if (display_service->camera->prefs.display_redraw==DISPLAY_REDRAW_ON) {
+	if (cam->prefs.display_redraw==DISPLAY_REDRAW_ON) {
 	  ConditionalTimeoutRedraw(display_service);
 	}
 	pthread_mutex_unlock(&display_service->mutex_data);
       }
     }
-    usleep(0);
+    usleep(100);
   }
   
   pthread_mutex_unlock(&info->mutex_cancel);
@@ -177,14 +174,14 @@ ConditionalTimeoutRedraw(chain_t* service)
   float interval;
   info=(displaythread_info_t*)service->data;
 
-  if (service->current_buffer->width!=-1) {
+  if (service->current_buffer->frame.size[0]!=-1) {
     info->redraw_current_time=times(&info->redraw_tms_buf);
     interval=fabs((float)(info->redraw_current_time-info->redraw_prev_time)/sysconf(_SC_CLK_TCK));
     if (interval>(1.0/service->camera->prefs.display_redraw_rate)) { // redraw e.g. 4 times per second
 #ifdef HAVE_SDLLIB
       if (SDL_LockYUVOverlay(info->sdloverlay) == 0) {
-	//MainStatus("Conditional display redraw");
-	convert_to_yuv_for_SDL(service->current_buffer, info->sdloverlay, preferences.overlay_byte_order);
+	//MainWarning("Conditional display redraw");
+	convert_to_yuv_for_SDL(&service->current_buffer->frame, info->sdloverlay, preferences.overlay_byte_order);
 	SDLDisplayArea(service);
 	SDL_UnlockYUVOverlay(info->sdloverlay);
 	SDL_DisplayYUVOverlay(info->sdloverlay, &info->sdlvideorect);
@@ -243,7 +240,7 @@ SDLInit(chain_t *display_service)
 
   // Initialize the SDL library (video subsystem)
   if ( SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE) == -1) {
-    fprintf(stderr,"Couldn't initialize SDL video subsystem\n");
+    Error("Couldn't initialize SDL video subsystem");
     return(0);
   }
   
@@ -251,8 +248,8 @@ SDLInit(chain_t *display_service)
   
   if ((xvinfo.max_width!=-1)&&(xvinfo.max_height!=-1)) {
     // if the XV area is too small, we use software accelleration
-    if ((xvinfo.max_width<display_service->current_buffer->width)||
-	(xvinfo.max_height<display_service->current_buffer->height)) {
+    if ((xvinfo.max_width<display_service->current_buffer->frame.size[0])||
+	(xvinfo.max_height<display_service->current_buffer->frame.size[1])) {
       //fprintf(stderr,"Using SW surface\n");
       info->sdlflags|= SDL_SWSURFACE;
       info->sdlflags&= ~SDL_HWSURFACE;
@@ -275,27 +272,27 @@ SDLInit(chain_t *display_service)
   modes=SDL_ListModes(NULL,info->sdlflags);
   if (modes!=(SDL_Rect**)-1) {
     // not all resolutions are OK for this video card. For safety we switch to software accel
-    fprintf(stderr,"No SDL mode available with hardware accel. Trying without HWSURFACE\n");
+    Warning("No SDL mode available with hardware accel. Trying without HWSURFACE");
     info->sdlflags&= ~SDL_HWSURFACE;
     info->sdlflags&= ~SDL_HWACCEL;
     modes=SDL_ListModes(NULL,info->sdlflags);
     if (modes!=(SDL_Rect**)-1) {
-      fprintf(stderr,"Still no modes available. Can't start SDL!\n");
+      Error("No video modes available, even without hardware acceleration. Can't start SDL!");
       SDL_Quit();
       return(0);
     }
   }
 
-  /*
+  
   // set coriander icon
-  icon_surface=SDL_CreateRGBSurfaceFrom((void*)coriander_logo_xpm)
-  SDL_WM_SetIcon(icon_surface,NULL);
-  */
+  //icon_surface=SDL_CreateRGBSurfaceFrom((void*)coriander_logo_xpm)
+  //SDL_WM_SetIcon(icon_surface,NULL);
+  
 
   info->sdlvideorect.x=0;
   info->sdlvideorect.y=0;
-  info->sdlvideorect.w=display_service->current_buffer->width;
-  info->sdlvideorect.h=display_service->current_buffer->height;
+  info->sdlvideorect.w=display_service->current_buffer->frame.size[0];
+  info->sdlvideorect.h=display_service->current_buffer->frame.size[1];
 
   // maximize display size to XV size if necessary
   if ((xvinfo.max_width!=-1)&&(xvinfo.max_height!=-1)) {
@@ -312,34 +309,34 @@ SDLInit(chain_t *display_service)
   info->sdlvideo = SDL_SetVideoMode(info->sdlvideorect.w, info->sdlvideorect.h, info->sdlbpp, info->sdlflags);
 
   if (info->sdlvideo == NULL) {
-    MainError(SDL_GetError());
+    Error(SDL_GetError());
     SDL_Quit();
     return(0);
   }
 
   if (SDL_SetColorKey( info->sdlvideo, SDL_SRCCOLORKEY, 0x0) < 0 ) {
-    MainError(SDL_GetError());
+    Error(SDL_GetError());
   }
   
   // Show cursor
   SDL_ShowCursor(1);
   
   // set window title:
-  SDL_WM_SetCaption(camera->prefs.name,camera->prefs.name);
+  SDL_WM_SetCaption(display_service->camera->prefs.name,display_service->camera->prefs.name);
 
   // this line broke everything for unknown reasons so I just remove it.
   //info->sdlvideo->format->BytesPerPixel=2;
 
   // Create YUV Overlay
   switch(preferences.overlay_byte_order) {
-  case OVERLAY_BYTE_ORDER_YUYV:
-    info->sdloverlay = SDL_CreateYUVOverlay(display_service->current_buffer->width,
-					    display_service->current_buffer->height,
+  case DC1394_BYTE_ORDER_YUYV:
+    info->sdloverlay = SDL_CreateYUVOverlay(display_service->current_buffer->frame.size[0],
+					    display_service->current_buffer->frame.size[1],
 					    SDL_YUY2_OVERLAY,info->sdlvideo);
     break;
-  case OVERLAY_BYTE_ORDER_UYVY:
-    info->sdloverlay = SDL_CreateYUVOverlay(display_service->current_buffer->width,
-					    display_service->current_buffer->height,
+  case DC1394_BYTE_ORDER_UYVY:
+    info->sdloverlay = SDL_CreateYUVOverlay(display_service->current_buffer->frame.size[0],
+					    display_service->current_buffer->frame.size[1],
 					    SDL_UYVY_OVERLAY,info->sdlvideo);
     break;
   default:
@@ -349,7 +346,7 @@ SDLInit(chain_t *display_service)
   
 
   if (info->sdloverlay==NULL) {
-    MainError(SDL_GetError());
+    Error(SDL_GetError());
     SDL_Quit();
     return(0);
   }
@@ -359,38 +356,6 @@ SDLInit(chain_t *display_service)
   return(1);
 }
 
-// we should optimize this for RGB too: RGB modes could use RGB-SDL instead of YUV overlay
-void
-convert_to_yuv_for_SDL(buffer_t *buffer, SDL_Overlay *sdloverlay, int overlay_byte_order)
-{
-  unsigned char *dest=sdloverlay->pixels[0];
-
-  switch(buffer->buffer_color_mode) {
-  case COLOR_FORMAT7_MONO8:
-    y2uyvy(buffer->image, dest, buffer->width, buffer->height, 
-	   sdloverlay->pitches[0], overlay_byte_order);
-    break;
-  case COLOR_FORMAT7_YUV411:
-    uyyvyy2uyvy(buffer->image,dest,buffer->width*buffer->height, overlay_byte_order);
-    break;
-  case COLOR_FORMAT7_YUV422:
-    yuyv2uyvy(buffer->image,dest,buffer->width*buffer->height, overlay_byte_order);
-    break;
-  case COLOR_FORMAT7_YUV444:
-    uyv2uyvy(buffer->image,dest,buffer->width*buffer->height, overlay_byte_order);
-    break;
-  case COLOR_FORMAT7_RGB8:
-    rgb2uyvy(buffer->image,dest,buffer->width*buffer->height, overlay_byte_order);
-    break;
-  case COLOR_FORMAT7_MONO16:
-    y162uyvy(buffer->image,dest,buffer->width*buffer->height,buffer->bpp, overlay_byte_order);
-    break;
-  case COLOR_FORMAT7_RGB16:
-    rgb482uyvy(buffer->image,dest,buffer->width*buffer->height, overlay_byte_order);
-    break;
-  }
-}
-
 
 void
 SDLDisplayArea(chain_t *display_service)
@@ -406,12 +371,12 @@ SDLDisplayArea(chain_t *display_service)
 
   pthread_mutex_lock(&watchthread_info.mutex_area);
   if (watchthread_info.draw==1) {
+    //fprintf(stderr,"drawing\n");
     upper_left[0]=watchthread_info.pos[0];
     upper_left[1]=watchthread_info.pos[1];
     lower_right[0]=watchthread_info.pos[0]+watchthread_info.size[0]-1;
     lower_right[1]=watchthread_info.pos[1]+watchthread_info.size[1]-1;
-    width=display_service->current_buffer->width;
-    pthread_mutex_unlock(&watchthread_info.mutex_area);
+    width=display_service->current_buffer->frame.size[0];
     
     if (lower_right[0]<upper_left[0]) {
       tmp=lower_right[0];
@@ -424,15 +389,17 @@ SDLDisplayArea(chain_t *display_service)
       upper_left[1]=tmp;
     }
 
+    //fprintf(stderr,"boundaries: %d %d %d %d\n",upper_left[0],upper_left[1],lower_right[0],lower_right[1]);
+
+    pimage=info->sdloverlay->pixels[0];//+i*info->sdloverlay->pitches[0];
     for (i=upper_left[1];i<=lower_right[1];i++) {
-      pimage=info->sdloverlay->pixels[0]+i*info->sdloverlay->pitches[0];
       for (j=upper_left[0];j<=lower_right[0];j++) {
-	pimage[j*2]=(unsigned char)(255-pimage[j*2]);
+	pimage[((i*width+j)<<1)]=(unsigned char)(255-pimage[((i*width+j)<<1)]);
+	pimage[((i*width+j)<<1)+1]=(unsigned char)(255-pimage[((i*width+j)<<1)+1]);
       }
     }
-  } else {
-    pthread_mutex_unlock(&watchthread_info.mutex_area);
-  }
+  } 
+  pthread_mutex_unlock(&watchthread_info.mutex_area);
 }
 
 void
@@ -440,45 +407,64 @@ SDLDisplayPattern(chain_t *display_service)
 {
   displaythread_info_t *info=(displaythread_info_t*)display_service->data;
   unsigned char *pimage;
-  int sx = display_service->current_buffer->width;
-  int sy = display_service->current_buffer->height;
-  int y,u,v;
+  int sx = display_service->current_buffer->frame.size[0];
+  int sy = display_service->current_buffer->frame.size[1];
+  int y,u,v,is,ie,js,je;
   unsigned char block[4];
   register int i;
   register int j;
 
-  RGB2YUV(camera->prefs.overlay_color_r,camera->prefs.overlay_color_g,camera->prefs.overlay_color_b,y,u,v);
+  is = (7*sx)/16; is = is-is%2;
+  ie = (9*sx)/16; ie = ie-ie%2;
+  js = (7*sy)/16;
+  je = (9*sy)/16;
+
+  RGB2YUV(display_service->camera->prefs.overlay_color_r,
+	  display_service->camera->prefs.overlay_color_g,
+	  display_service->camera->prefs.overlay_color_b,y,u,v);
 
   switch(display_service->camera->prefs.overlay_type) {
   case OVERLAY_TYPE_REPLACE:
     switch(preferences.overlay_byte_order) {
-    case OVERLAY_BYTE_ORDER_YUYV:
+    case DC1394_BYTE_ORDER_YUYV:
       block[0]=y;
       block[1]=u;
       block[2]=y;
       block[3]=v;
       break;
-    case OVERLAY_BYTE_ORDER_UYVY:
+    case DC1394_BYTE_ORDER_UYVY:
       block[0]=u;
       block[1]=y;
       block[2]=v;
       block[3]=y;
       break;
     default:
-      fprintf(stderr,"Invalid overlay coding\n");
+      fprintf(stderr,"Invalid overlay byte order coding: %d\n",preferences.overlay_byte_order);
       break;
     }
     break;
   case OVERLAY_TYPE_RANDOM:
-    block[0]=random()/(RAND_MAX/255);
-    block[1]=random()/(RAND_MAX/255);
-    block[2]=random()/(RAND_MAX/255);
-    block[3]=random()/(RAND_MAX/255);
+    switch(preferences.overlay_byte_order) {
+    case DC1394_BYTE_ORDER_YUYV:
+      block[0]=random()/(RAND_MAX/255);
+      block[1]=random()/(RAND_MAX/255);
+      block[2]=block[0];
+      block[3]=random()/(RAND_MAX/255);
+      break;
+    case DC1394_BYTE_ORDER_UYVY:
+      block[0]=random()/(RAND_MAX/255);
+      block[1]=random()/(RAND_MAX/255);
+      block[2]=random()/(RAND_MAX/255);
+      block[3]=block[1];
+      break;
+    }
     break;
   case OVERLAY_TYPE_INVERT:
+    memset(block, 0, sizeof(block));
     break;
   default:
-    fprintf(stderr,"Invalid overlay coding\n");
+    fprintf(stderr,"Invalid display overlay type: %d\n",display_service->camera->prefs.overlay_type);
+    memset(block, 0, sizeof(block));
     break;
   }
 
@@ -489,163 +475,192 @@ SDLDisplayPattern(chain_t *display_service)
     break;
   case OVERLAY_PATTERN_RECTANGLE:
     if (display_service->camera->prefs.overlay_type==OVERLAY_TYPE_INVERT) {
-      j=(7*sy)/16;
-      j=j-j%4;
-      for (i=(7*sx)/16;i<=(9*sx)/16;i+=2) {
-	INVPIX(pimage, (j*sx+i)*2);
-	INVPIX(pimage, ((j-1)*sx+i)*2);
+      j=js;
+      for (i=is;i<=ie;i+=2) {
+	INVPIX(pimage, ((j*sx+i)<<1));
+	INVPIX(pimage, (((j-1)*sx+i)<<1));
       }
-      j=(9*sy)/16;
-      j=j-j%4;
-      for (i=(7*sx)/16;i<=(9*sx)/16;i+=2) {
-	INVPIX(pimage, (j*sx+i)*2);
-	INVPIX(pimage, ((j+1)*sx+i)*2);
+      j=je;
+      for (i=is;i<=ie;i+=2) {
+	INVPIX(pimage,  ((j*sx+i)<<1));
+	INVPIX(pimage, (((j+1)*sx+i)<<1));
       }
-      j=(7*sx)/16;
-      j=j-j%4;
-      for (i=(7*sy)/16;i<=(9*sy)/16;i++) {
-	INVPIX(pimage, (i*sx+j)*2);
+      i=is;
+      for (j=js;j<=je;j++) {
+	INVPIX(pimage, ((j*sx+i)<<1));
       }
-      j=(9*sx)/16;
-      j=j-j%4;
-      for (i=(7*sy)/16;i<=(9*sy)/16;i++) {
-	INVPIX(pimage, (i*sx+j)*2);
+      i=ie;
+      for (j=js;j<=je;j++) {
+	INVPIX(pimage, ((j*sx+i)<<1));
       }
     }
     else {
-      j=(7*sy)/16;
-      j=j-j%4;
-      for (i=(7*sx)/16;i<=(9*sx)/16;i+=2) {
-	REPLPIX(pimage, block, (j*sx+i)*2);
-	REPLPIX(pimage, block, ((j-1)*sx+i)*2);
+      j=js;
+      for (i=is;i<=ie;i+=2) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
+	REPLPIX(pimage, block, (((j-1)*sx+i)<<1));
       }
-      j=(9*sy)/16;
-      j=j-j%4;
-      for (i=(7*sx)/16;i<=(9*sx)/16;i+=2) {
-	REPLPIX(pimage, block, (j*sx+i)*2);
-	REPLPIX(pimage, block, ((j+1)*sx+i)*2);
+      j=je;
+      for (i=is;i<=ie;i+=2) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
+	REPLPIX(pimage, block, (((j+1)*sx+i)<<1));
       }
-      j=(7*sx)/16;
-      j=j-j%4;
-      for (i=(7*sy)/16;i<=(9*sy)/16;i++) {
-	REPLPIX(pimage, block, (i*sx+j)*2);
+      i=is;
+      for (j=js;j<=je;j++) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
       }
-      j=(9*sx)/16;
-      j=j-j%4;
-      for (i=(7*sy)/16;i<=(9*sy)/16;i++) {
-	REPLPIX(pimage, block, (i*sx+j)*2);
+      i=ie;
+      for (j=js;j<=je;j++) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
       }
     }
     break;
   case OVERLAY_PATTERN_SMALL_CROSS:
     if (display_service->camera->prefs.overlay_type==OVERLAY_TYPE_INVERT) {
-      j=sy/2;
-      j=j-j%4;
-      for (i=(7*sx)/16;i<=(9*sx)/16;i+=2) {
-	INVPIX(pimage, (j*sx+i)*2);
-	INVPIX(pimage, ((j-1)*sx+i)*2);
+      j=sy/2; 
+      for (i=is;i<=ie;i+=2) {
+	INVPIX(pimage, ((j*sx+i)<<1));
+	INVPIX(pimage, (((j-1)*sx+i)<<1));
       }
-      j=sx/2;
-      j=j-j%4;
-      for (i=(7*sy)/16;i<=(9*sy)/16;i++) {
-	INVPIX(pimage, (i*sx+j)*2);
+      i=sx/2; i=i-i%2;
+      for (j=js;j<=je;j++) {
+	INVPIX(pimage, ((j*sx+i)<<1));
       }
     }
     else {
       j=sy/2;
-      j=j-j%4;
-      for (i=(7*sx)/16;i<=(9*sx)/16;i+=2) {
-	REPLPIX(pimage, block, (j*sx+i)*2);
-	REPLPIX(pimage, block, ((j-1)*sx+i)*2);
+      for (i=is;i<=ie;i+=2) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
+	REPLPIX(pimage, block, (((j-1)*sx+i)<<1));
       }
-      j=sx/2;
-      j=j-j%4;
-      for (i=(7*sy)/16;i<=(9*sy)/16;i++) {
-	REPLPIX(pimage, block, (i*sx+j)*2);
+      i=sx/2; i=i-i%2;
+      for (j=js;j<=je;j++) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
       }
     }
     break;
   case OVERLAY_PATTERN_LARGE_CROSS:
     if (display_service->camera->prefs.overlay_type==OVERLAY_TYPE_INVERT) {
       j=sy/2;
-      j=j-j%4;
       for (i=0;i<sx;i+=2) {
-	INVPIX(pimage, (j*sx+i)*2);
-	INVPIX(pimage, ((j-1)*sx+i)*2);
+	INVPIX(pimage, ((j*sx+i)<<1));
+	INVPIX(pimage, (((j-1)*sx+i)<<1));
       }
-      j=sx/2;
-      j=j-j%4;
-      for (i=0;i<sy;i++) {
-	INVPIX(pimage, (i*sx+j)*2);
+      i=sx/2; i=i-i%2;
+      for (j=0;j<sy;j++) {
+	INVPIX(pimage, ((j*sx+i)<<1));
       }
     }
     else {
       j=sy/2;
-      j=j-j%4;
       for (i=0;i<sx;i+=2) {
-	REPLPIX(pimage, block, (j*sx+i)*2);
-	REPLPIX(pimage, block, ((j-1)*sx+i)*2);
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
+	REPLPIX(pimage, block, (((j-1)*sx+i)<<1));
       }
-      j=sx/2;
-      j=j-j%4;
-      for (i=0;i<sy;i++) {
-	REPLPIX(pimage, block, (i*sx+j)*2);
+      i=sx/2; i=i-i%2;
+      for (j=0;j<sy;j++) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
       }
     }
     break;
   case OVERLAY_PATTERN_GOLDEN_MEAN:
     if (display_service->camera->prefs.overlay_type==OVERLAY_TYPE_INVERT) {
       j=sy/3;
-      j=j-j%4;
       for (i=0;i<sx;i+=2) {
-	INVPIX(pimage, (j*sx+i)*2);
-	INVPIX(pimage, ((j-1)*sx+i)*2);
+	INVPIX(pimage, ((j*sx+i)<<1));
+	INVPIX(pimage, (((j-1)*sx+i)<<1));
       }
-      j=2*sy/3;
-      j=j-j%4;
+      j=(2*sy)/3;
       for (i=0;i<sx;i+=2) {
-	INVPIX(pimage, (j*sx+i)*2);
-	INVPIX(pimage, ((j-1)*sx+i)*2);
+	INVPIX(pimage, ((j*sx+i)<<1));
+	INVPIX(pimage, (((j-1)*sx+i)<<1));
       }
-      j=sx/3;
-      j=j-j%4;
-      for (i=0;i<sy;i++) {
-	INVPIX(pimage, (i*sx+j)*2);
+      i=sx/3;
+      i=i-i%2;
+      for (j=0;j<sy;j++) {
+	INVPIX(pimage, ((j*sx+i)<<1));
       }
-      j=2*sx/3;
-      j=j-j%4;
-      for (i=0;i<sy;i++) {
-	INVPIX(pimage, (i*sx+j)*2);
+      i=(2*sx)/3;
+      i=i-i%2;
+      for (j=0;j<sy;j++) {
+	INVPIX(pimage, ((j*sx+i)<<1));
       }
     }
     else {
       j=sy/3;
-      j=j-j%4;
       for (i=0;i<sx;i+=2) {
-	REPLPIX(pimage, block, (j*sx+i)*2);
-	REPLPIX(pimage, block, ((j-1)*sx+i)*2);
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
+	REPLPIX(pimage, block, (((j-1)*sx+i)<<1));
       }
-      j=2*sy/3;
-      j=j-j%4;
+      j=(2*sy)/3;
       for (i=0;i<sx;i+=2) {
-	REPLPIX(pimage, block, (j*sx+i)*2);
-	REPLPIX(pimage, block, ((j-1)*sx+i)*2);
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
+	REPLPIX(pimage, block, (((j-1)*sx+i)<<1));
       }
-      j=sx/3;
-      j=j-j%4;
-      for (i=0;i<sy;i++) {
-	REPLPIX(pimage, block, (i*sx+j)*2);
+      i=sx/3;
+      i=i-i%2;
+      for (j=0;j<sy;j++) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
       }
-      j=2*sx/3;
-      j=j-j%4;
-      for (i=0;i<sy;i++) {
-	REPLPIX(pimage, block, (i*sx+j)*2);
+      i=(2*sx)/3;
+      i=i-i%2;
+      for (j=0;j<sy;j++) {
+	REPLPIX(pimage, block, ((j*sx+i)<<1));
       }
-      
     }
     break;
   case OVERLAY_PATTERN_IMAGE:
     break;
+  case OVERLAY_PATTERN_OVER_UNDEREXPOSED:
+    switch(preferences.overlay_byte_order) {
+    case DC1394_BYTE_ORDER_YUYV:
+      if (display_service->camera->prefs.overlay_type==OVERLAY_TYPE_INVERT) {
+	for (i=0;i<sx;i+=2) {
+	  for (j=0;j<sy;j++) {
+	    if ((pimage[(j*sx+i)<<1]>254)||(pimage[(j*sx+i)<<1]<1)||
+		(pimage[(j*sx+i+1)<<1]>254)||(pimage[(j*sx+i+1)<<1]<1)) {
+	      INVPIX(pimage, ((j*sx+i)<<1));
+	    }
+	  }
+	}
+      }
+      else {
+	for (i=0;i<sx;i+=2) {
+	  for (j=0;j<sy;j++) {
+	    if ((pimage[(j*sx+i)<<1]>254)||(pimage[(j*sx+i)<<1]<1)||
+		(pimage[(j*sx+i+1)<<1]>254)||(pimage[(j*sx+i+1)<<1]<1)) {
+	      REPLPIX(pimage, block, ((j*sx+i)<<1));
+	    }
+	  }
+	}
+      }
+      break;
+    case DC1394_BYTE_ORDER_UYVY:
+      if (display_service->camera->prefs.overlay_type==OVERLAY_TYPE_INVERT) {
+	for (i=0;i<sx;i+=2) {
+	  for (j=0;j<sy;j++) {
+	    if ((pimage[((j*sx+i)<<1)+1]>254)||(pimage[((j*sx+i)<<1)+1]<1)||
+		(pimage[((j*sx+i+1)<<1)+1]>254)||(pimage[((j*sx+i+1)<<1)+1]<1)) {
+	      INVPIX(pimage, ((j*sx+i)<<1));
+	    }
+	  }
+	}
+      }
+      else {
+	for (i=0;i<sx;i+=2) {
+	  for (j=0;j<sy;j++) {
+	    if ((pimage[((j*sx+i)<<1)+1]>254)||(pimage[((j*sx+i)<<1)+1]<1)||
+		(pimage[((j*sx+i+1)<<1)+1]>254)||(pimage[((j*sx+i+1)<<1)+1]<1)) {
+	      REPLPIX(pimage, block, ((j*sx+i)<<1));
+	    }
+	  }
+	}
+      }
+      break;
+    default:
+      fprintf(stderr,"Invalid overlay byte order\n");
+    }
+    break;
   default:
     fprintf(stderr,"Wrong overlay pattern ID\n");
     break;
@@ -661,7 +676,7 @@ SDLQuit(chain_t *display_service)
 
 #ifdef HAVE_SDLLIB
   // if width==-1, SDL was never initialized so we do nothing
-  if (display_service->current_buffer->width!=-1) {
+  if (display_service->current_buffer->frame.size[0]!=-1) {
     SDLEventStopThread(display_service);
     SDL_FreeYUVOverlay(info->sdloverlay);
     SDL_FreeSurface(info->sdlvideo);
@@ -676,70 +691,64 @@ DisplayThreadCheckParams(chain_t *display_service)
 
   displaythread_info_t *info;
   int first_time=0;
-  int size_change;
+  //int size_change;
   int prev_image_size[2];
   int prev_overlay_size[2];
   info=(displaythread_info_t*)display_service->data;
   
-  // copy harmless parameters anyway:
-  display_service->local_param_copy.bpp=display_service->current_buffer->bpp;
-  display_service->local_param_copy.bayer_pattern=display_service->current_buffer->bayer_pattern;
-  if (display_service->current_buffer->width==-1)
+  if (display_service->current_buffer->frame.size[0]==-1)
     fprintf(stderr,"Error: display size: %dx%d\n",
-	    display_service->current_buffer->width,
-	    display_service->current_buffer->height);
+	    display_service->current_buffer->frame.size[0],
+	    display_service->current_buffer->frame.size[1]);
 
-  // if some parameters changed, we need to re-allocate the local buffers and restart the display
-  if ((display_service->current_buffer->width!=display_service->local_param_copy.width)||
-      (display_service->current_buffer->height!=display_service->local_param_copy.height)||
-      (display_service->current_buffer->bytes_per_frame!=display_service->local_param_copy.bytes_per_frame)||
-      (display_service->current_buffer->mode!=display_service->local_param_copy.mode)||
-      (display_service->current_buffer->format!=display_service->local_param_copy.format)||
-      // check F7 color mode change
-      ((display_service->current_buffer->format==FORMAT_SCALABLE_IMAGE_SIZE)&&
-       (display_service->current_buffer->format7_color_mode!=display_service->local_param_copy.format7_color_mode)
-       ) ||
-      // check bayer and stereo decoding
-      (display_service->current_buffer->stereo_decoding!=display_service->local_param_copy.stereo_decoding)||
-      (display_service->current_buffer->bayer!=display_service->local_param_copy.bayer)
-      ) {
+  //fprintf(stderr,"check params\n");
+  // if some parameters changed, we need to restart the display
+  //fprintf(stderr,"Before: [%d %d] After: [%d %d]\n",
+  //	  display_service->local_param_copy.frame.size[0],display_service->local_param_copy.frame.size[1],
+  //	  display_service->current_buffer->frame.size[0],display_service->current_buffer->frame.size[1]);
+  if ((display_service->current_buffer->frame.size[0]!=display_service->local_param_copy.frame.size[0])||
+      (display_service->current_buffer->frame.size[1]!=display_service->local_param_copy.frame.size[1])   ) {
 
-    first_time=((display_service->local_param_copy.width==-1)&&(display_service->current_buffer->width!=-1));
-    size_change=((display_service->current_buffer->width!=display_service->local_param_copy.width)||
-		 (display_service->current_buffer->height!=display_service->local_param_copy.height));
-    
-    prev_image_size[0]=display_service->local_param_copy.width;
-    prev_image_size[1]=display_service->local_param_copy.height;
+    first_time=((display_service->local_param_copy.frame.size[0]==-1)&&(display_service->current_buffer->frame.size[0]!=-1));
+    //if (first_time>0)
+    //fprintf(stderr,"  first frame...\n");
+      
+    prev_image_size[0]=display_service->local_param_copy.frame.size[0];
+    prev_image_size[1]=display_service->local_param_copy.frame.size[1];
+
+    // alternating between thos two methods may give a clue (don't forget deal with the 'else memcpy' below) ///////////////////////////////////////////
+    memcpy(&display_service->local_param_copy,display_service->current_buffer,sizeof(buffer_t));
+    //display_service->local_param_copy.frame.size[0]=display_service->current_buffer->frame.size[0];
+    //display_service->local_param_copy.frame.size[1]=display_service->current_buffer->frame.size[1];
 
-    // copy all new parameters:
-    display_service->local_param_copy.width=display_service->current_buffer->width;
-    display_service->local_param_copy.height=display_service->current_buffer->height;
-    display_service->local_param_copy.bytes_per_frame=display_service->current_buffer->bytes_per_frame;
-    display_service->local_param_copy.mode=display_service->current_buffer->mode;
-    display_service->local_param_copy.format=display_service->current_buffer->format;
-    display_service->local_param_copy.format7_color_mode=display_service->current_buffer->format7_color_mode;
-    display_service->local_param_copy.stereo_decoding=display_service->current_buffer->stereo_decoding;
-    display_service->local_param_copy.bayer=display_service->current_buffer->bayer;
-    display_service->local_param_copy.buffer_image_bytes=display_service->current_buffer->buffer_image_bytes;
-    
     // DO SOMETHING
+    //fprintf(stderr,"  fs: %d\n",display_service->local_param_copy.frame.size[0]);
     // if the width is not -1, that is if some image has already reached the thread and the size has changed
-    if ((display_service->local_param_copy.width!=-1)&&(size_change!=0)) {
+    if (display_service->local_param_copy.frame.size[0]!=-1) { //&&(size_change!=0)) {
+      //fprintf(stderr,"   do something...\n");
       if (first_time) {
 	SDLInit(display_service);
       } else {
+
+	//usleep(1000000);
+
 	// note: in order to preserve the previous scaling and ratio, the previous parameters are used to
 	// determine the new size of the display area
 	prev_overlay_size[0]=info->sdlvideorect.w;
 	prev_overlay_size[1]=info->sdlvideorect.h;
 	watchthread_info.draw=0;
+	//SDLEventStopThread(display_service);
 	SDLResizeDisplay(display_service,
-			 display_service->current_buffer->width*prev_overlay_size[0]/prev_image_size[0],
-			 display_service->current_buffer->height*prev_overlay_size[1]/prev_image_size[1]);
+			 display_service->current_buffer->frame.size[0]*prev_overlay_size[0]/prev_image_size[0],
+			 display_service->current_buffer->frame.size[1]*prev_overlay_size[1]/prev_image_size[1]);
+	//SDLEventStartThread(display_service);
       }
     }
+    //fprintf(stderr,"SDL updated.\n");
   }
 
+  memcpy(&display_service->local_param_copy,display_service->current_buffer,sizeof(buffer_t));
+  display_service->local_param_copy.frame.allocated_image_bytes=0;
 }
 
 #endif
