@@ -27,32 +27,19 @@ typedef enum
 
 typedef struct
 { 
-  raw1394handle_t         handle;
-  receive_method_t        receive_method;
-  dc1394_cameracapture    capture;
-  int video1394_dropframes;
-  char video1394_device[STRING_SIZE];
-
-  unsigned char *temp;
-  long long unsigned int temp_size;
-  int temp_allocated;
+  pthread_mutex_t    mutex_cancel;
+  int                cancel_req;
 
+  dc1394video_frame_t tempframe;
   int orig_sizex;
   int orig_sizey;
   int cond16bit;
 
-  int dma_buffer_size;
-
-  struct timeval rawtime;
-
 } isothread_info_t;
 
 gint
 IsoStartThread(camera_t* cam);
 
-void*
-IsoCleanupThread(void* arg);
-
 void*
 IsoThread(void* arg);
 
@@ -69,7 +56,7 @@ void
 AllocImageBuffer(chain_t* iso_service);
 
 void
-SetColorMode(buffer_t *buffer);
+SetColorMode(int mode, buffer_t *buffer, int f7_color);
 
 #endif // __THREAD_ISO_H__
 
