@@ -51,23 +51,6 @@ SaveStartThread(camera_t* cam)
     info->frame.allocated_image_bytes=0;
     info->bigbuffer=NULL;
 
-    if ((cam->prefs.use_ram_buffer==TRUE)&&
-	((cam->prefs.save_format==SAVE_FORMAT_RAW_VIDEO)||
-#ifdef HAVE_FFMPEG
-	 (cam->prefs.save_format==SAVE_FORMAT_MPEG)||
-#endif
-	 (cam->prefs.save_format==SAVE_FORMAT_PVN))) {
-      info->bigbuffer_position=0;
-      info->bigbuffer=(unsigned char*)malloc(cam->prefs.ram_buffer_size*1024*1024*sizeof(unsigned char));
-      if (info->bigbuffer==NULL) {
-	Error("Could not allocate memory for RAM buffer save service");
-	pthread_mutex_unlock(&save_service->mutex_data);
-	FreeChain(save_service);
-	save_service=NULL;
-	return(-1);
-      }
-    }
-
     /* Insert chain and start service*/
     pthread_mutex_lock(&save_service->mutex_struct);
     InsertChain(cam, save_service);
@@ -250,6 +233,9 @@ GetSaveFD(chain_t *save_service, FILE **fd, char *filename_out)
   case SAVE_FORMAT_RAW:
 #ifdef HAVE_FFMPEG
   case SAVE_FORMAT_JPEG:
+#endif
+#ifdef HAVE_LIBTIFF
+  case SAVE_FORMAT_TIFF:
 #endif
     // first handle the case of save-to-dir
     if (cam->prefs.save_to_dir==0) {
@@ -352,6 +338,16 @@ GetSaveFD(chain_t *save_service, FILE **fd, char *filename_out)
   case SAVE_FORMAT_JPEG:
     // do nothing
     break;
+#endif
+#ifdef HAVE_LIBTIFF // note that we return a TIFF* disguised as a FILE* here. Ugly.
+  case SAVE_FORMAT_TIFF:
+    ProtectFilename(filename_out);
+    *fd=(FILE*)((void*)TIFFOpen(filename_out,"w"));
+    if (!fd) {
+      fprintf(stderr,"Can't create sequence file for saving\n");
+      return DC1394_FAILURE;
+    }
+    break;
 #endif
   default:
     fprintf(stderr,"unsupported format!\n");
@@ -384,7 +380,7 @@ InitVideoFile(chain_t *save_service, FILE *fd, char *filename_out)
 #endif
 
   // (JG) if extension is PVN, write PVN header here
-  if ((cam->prefs.save_format==SAVE_FORMAT_PVN) && (cam->prefs.use_ram_buffer==FALSE)) {//-----------------------------------
+  if (cam->prefs.save_format==SAVE_FORMAT_PVN) {//-----------------------------------
     //fprintf(stderr,"pvn header write\n");
     dc1394_framerate_as_float(framerate, &fps);
     writePVNHeader(fd, save_service->current_buffer->frame.color_coding,
@@ -395,7 +391,7 @@ InitVideoFile(chain_t *save_service, FILE *fd, char *filename_out)
   }
 
 #ifdef HAVE_FFMPEG
-  if ((cam->prefs.save_format==SAVE_FORMAT_MPEG) && (cam->prefs.use_ram_buffer==FALSE)) {//-----------------------------------
+  if (cam->prefs.save_format==SAVE_FORMAT_MPEG) {//-----------------------------------
     // MPEG
     //fprintf(stderr,"setting up mpeg codec\n");
     //video_encode_init(save_service->current_buffer->width,
@@ -504,31 +500,6 @@ InitVideoFile(chain_t *save_service, FILE *fd, char *filename_out)
   return DC1394_SUCCESS;
 }
 
-void
-FillRamBuffer(chain_t *save_service)
-{
-  savethread_info_t *info;
-
-  info=(savethread_info_t*)save_service->data;
-  camera_t *cam=save_service->camera;
-
-
-  if ((cam->prefs.use_ram_buffer==TRUE)&&
-      ((cam->prefs.save_format==SAVE_FORMAT_RAW_VIDEO)||
-#ifdef HAVE_FFMPEG
-       (cam->prefs.save_format==SAVE_FORMAT_MPEG)||
-#endif
-       (cam->prefs.save_format==SAVE_FORMAT_PVN))) {
-    if (cam->prefs.ram_buffer_size*1024*1024-info->bigbuffer_position>=save_service->current_buffer->frame.image_bytes) {
-      memcpy(&info->bigbuffer[info->bigbuffer_position], save_service->current_buffer->frame.image, save_service->current_buffer->frame.image_bytes);
-      info->bigbuffer_position+=save_service->current_buffer->frame.image_bytes;
-    }
-    else { // buffer is full, exit thread
-      info->cancel_req=1;
-    }
-  }
-}
-
 void
 SavePPM(chain_t *save_service, FILE *fd)
 {
@@ -638,7 +609,7 @@ SaveJPEGFrame(chain_t *save_service, char *filename_out)
   info=(savethread_info_t*)save_service->data;
   //if (save_service->current_buffer->frame.color_coding!=DC1394_COLOR_CODING_YUV411)&&(
 
-  fprintf(stderr,"Trying to save a JPEG frame...\n");
+  //fprintf(stderr,"Trying to save a JPEG frame...\n");
   
   info->picture = alloc_picture(PIX_FMT_YUVJ420P, save_service->current_buffer->frame.size[0], save_service->current_buffer->frame.size[1]);
   if (!info->picture) {
@@ -681,6 +652,88 @@ SaveJPEGFrame(chain_t *save_service, char *filename_out)
 }
 #endif
 
+#ifdef HAVE_LIBTIFF
+
+void
+SaveTIFF(chain_t *save_service, TIFF* tif)
+{
+    unsigned int bpp, spp, i,w,h,depth;
+    savethread_info_t *info;
+    unsigned char *src;
+    info=(savethread_info_t*)save_service->data;
+
+    w=save_service->current_buffer->frame.size[0];
+    h=save_service->current_buffer->frame.size[1];
+    depth=save_service->current_buffer->frame.data_depth;
+
+    switch (save_service->current_buffer->frame.color_coding) {
+    case DC1394_COLOR_CODING_MONO8:
+    case DC1394_COLOR_CODING_RAW8:
+        bpp=1; spp=1;
+        src=save_service->current_buffer->frame.image;
+        break;
+    case DC1394_COLOR_CODING_MONO16:
+    case DC1394_COLOR_CODING_RAW16:
+    case DC1394_COLOR_CODING_MONO16S:
+        bpp=2; spp=1;
+        src=save_service->current_buffer->frame.image;
+        break;
+    case DC1394_COLOR_CODING_YUV411:
+    case DC1394_COLOR_CODING_YUV422:
+    case DC1394_COLOR_CODING_YUV444:
+        bpp=1; spp=3;
+        convert_to_rgb(&save_service->current_buffer->frame, &info->frame);
+        src=info->frame.image;
+        break;
+    case DC1394_COLOR_CODING_RGB8:
+        bpp=1; spp=3;
+        src=save_service->current_buffer->frame.image;
+        break;
+    case DC1394_COLOR_CODING_RGB16:
+        bpp=2; spp=3;
+        src=save_service->current_buffer->frame.image;
+        break;
+    default:
+        fprintf(stderr, "Unknown buffer format!\n");
+        return;
+    }
+    TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, w);
+    TIFFSetField(tif, TIFFTAG_IMAGELENGTH, h);
+    TIFFSetField(tif, TIFFTAG_ORIENTATION, ORIENTATION_TOPLEFT);
+    TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, spp);
+    TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, depth);
+    TIFFSetField(tif, TIFFTAG_SOFTWARE, "Coriander");
+    TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
+    TIFFSetField(tif, TIFFTAG_MINSAMPLEVALUE, 0);
+    TIFFSetField(tif, TIFFTAG_MAXSAMPLEVALUE, (2^depth)-1);
+    TIFFSetField(tif, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB);
+    switch (spp) {
+    case 1: 
+        TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
+        break;
+    case 3:
+        TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
+        TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
+        break;
+    default:
+        fprintf(stderr,"Invalid number of TIFF fields\n");
+    }
+    // byte swap, 'cause TIFF doesn't seem to do this right.
+    if (bpp==2) {
+        uint8_t a;
+        for (i=0;i<h*spp*w;i++) {
+            a=src[2*i];
+            src[2*i]=src[2*i+1];
+            src[2*i+1]=a;
+        }
+    }
+    for (i=0;i<h;i++)
+        if (TIFFWriteScanline(tif, &src[i*w*bpp*spp], i, 0)<0)
+            fprintf(stderr,"Error writing TIFF line %d\n",i);
+
+    TIFFClose(tif);
+}
+#endif
 
 void*
 SaveThread(void* arg)
@@ -691,8 +744,7 @@ SaveThread(void* arg)
   //GdkImlibImage *im=NULL; // V20***
   long int skip_counter;
   FILE *fd=NULL;
-  float tmp, fps;
-  int i;
+  float tmp;
   unsigned char* tmp_buf=NULL;
 
   filename_out=(char*)malloc(STRING_SIZE*sizeof(char));
@@ -760,18 +812,8 @@ SaveThread(void* arg)
 	      }
 	    }
 
-	    // rambuffer operation
-	    if ((cam->prefs.use_ram_buffer==TRUE)&&
-		((cam->prefs.save_format==SAVE_FORMAT_RAW_VIDEO)||
-#ifdef HAVE_FFMPEG
-		 (cam->prefs.save_format==SAVE_FORMAT_MPEG)||
-#endif
-		 (cam->prefs.save_format==SAVE_FORMAT_PVN))) {
-	      FillRamBuffer(save_service);
-	    }
-	    else { // normal operation (no RAM buffer)
-	      switch (cam->prefs.save_format) {
-	      case SAVE_FORMAT_RAW:
+            switch (cam->prefs.save_format) {
+            case SAVE_FORMAT_RAW:
 		//fprintf(stderr,"writing raw...");
 		fwrite(save_service->current_buffer->frame.image, save_service->current_buffer->frame.image_bytes, 1, fd);
 		//fprintf(stderr,"done. closing fd...");
@@ -779,44 +821,47 @@ SaveThread(void* arg)
 		fd=NULL;
 		//fprintf(stderr,"done\n");
 		break;
-	      case SAVE_FORMAT_RAW_VIDEO:
+            case SAVE_FORMAT_RAW_VIDEO:
 		fwrite(save_service->current_buffer->frame.image, save_service->current_buffer->frame.image_bytes, 1, fd);
 		break;
-	      case SAVE_FORMAT_PVN:
+            case SAVE_FORMAT_PVN:
 		if (needsConversionForPVN(save_service->current_buffer->frame.color_coding)>0) {
-		  // we assume that if it needs conversion, the output of the conversion is an 8bpp RGB
-		  tmp_buf = (unsigned char*)malloc(3*save_service->current_buffer->frame.size[0]*save_service->current_buffer->frame.size[1]*sizeof(unsigned char));
-		  convert_for_pvn(save_service->current_buffer->frame.image, save_service->current_buffer->frame.size[0],
-				  save_service->current_buffer->frame.size[1], 0, save_service->current_buffer->frame.color_coding, tmp_buf);
-		  fwrite(tmp_buf, 3*save_service->current_buffer->frame.size[0]*save_service->current_buffer->frame.size[1], 1, fd);
-		  free(tmp_buf);
-		  tmp_buf=NULL;
+                    // we assume that if it needs conversion, the output of the conversion is an 8bpp RGB
+                    tmp_buf = (unsigned char*)malloc(3*save_service->current_buffer->frame.size[0]*save_service->current_buffer->frame.size[1]*sizeof(unsigned char));
+                    convert_for_pvn(save_service->current_buffer->frame.image, save_service->current_buffer->frame.size[0],
+                                    save_service->current_buffer->frame.size[1], 0, save_service->current_buffer->frame.color_coding, tmp_buf);
+                    fwrite(tmp_buf, 3*save_service->current_buffer->frame.size[0]*save_service->current_buffer->frame.size[1], 1, fd);
+                    free(tmp_buf);
+                    tmp_buf=NULL;
 		}
 		else {
-		  // no conversion, we can dump the data
-		  fwrite(save_service->current_buffer->frame.image, save_service->current_buffer->frame.image_bytes, 1, fd);
+                    // no conversion, we can dump the data
+                    fwrite(save_service->current_buffer->frame.image, save_service->current_buffer->frame.image_bytes, 1, fd);
 		}
 		break;
 #ifdef HAVE_FFMPEG
-	      case SAVE_FORMAT_MPEG:
+            case SAVE_FORMAT_MPEG:
 		// video saving mode
 		//fprintf(stderr,"entering MPEG save and convert section\n");
 		SaveMPEGFrame(save_service);
 		break;
-	      case SAVE_FORMAT_JPEG:
+            case SAVE_FORMAT_JPEG:
 		/* Save JPEG file using FFMPEG... Much, much faster... There is no need for YUV->RGB color space conversions */
 		SaveJPEGFrame(save_service, filename_out);
 		break;
 #endif
-	      case SAVE_FORMAT_PPM:
+            case SAVE_FORMAT_PPM:
 		SavePPM(save_service,fd);
 		fclose(fd);
 		fd=NULL;
 		break;
-	      default:
+            case SAVE_FORMAT_TIFF:
+                SaveTIFF(save_service, (TIFF*)((void*)(fd)));
+                fd=NULL;
+                break;
+            default:
 		fprintf(stderr,"Unsupported file format\n");
-	      } // end save format switch
-	    } // end ram buffer if
+            } // end save format switch
 	    save_service->processed_frames++;
 	  }
 	  else
@@ -857,54 +902,6 @@ SaveThread(void* arg)
     //fprintf(stderr,"??");
     usleep(100);
   }
-
-  // we now have to close the video file properly and handle ram buffer operation
-
-  //fprintf(stderr,"Break completed\n");
-
-  dc1394framerate_t framerate;
-  if (cam->prefs.use_ram_buffer==TRUE) {
-    switch(cam->prefs.save_format) {
-    case SAVE_FORMAT_RAW_VIDEO:
-#ifdef HAVE_FFMPEG
-    case SAVE_FORMAT_MPEG:
-      fwrite(info->bigbuffer, info->bigbuffer_position, 1, fd);
-      break;
-#endif
-    case SAVE_FORMAT_PVN:
-      dc1394_video_get_framerate(cam->camera_info,&framerate);
-      dc1394_framerate_as_float(framerate, &fps);
-      writePVNHeader(fd, save_service->current_buffer->frame.color_coding,
-		     save_service->current_buffer->frame.size[0],
-		     save_service->current_buffer->frame.size[1],
-		     getDepth(info->bigbuffer_position, save_service->current_buffer->frame.color_coding, 
-			      save_service->current_buffer->frame.size[1], save_service->current_buffer->frame.size[0]),
-		     getConvertedBytesPerChannel(save_service->current_buffer->frame.color_coding)*8,
-		     fps);
-      
-      if(needsConversionForPVN(save_service->current_buffer->frame.color_coding)==FALSE) {
-	fwrite(info->bigbuffer, info->bigbuffer_position, 1, fd);
-      }
-      else {
-	// we assume that if it needs conversion, the output of the conversion is an 8bpp RGB
-	tmp_buf = (unsigned char*)malloc(3*save_service->current_buffer->frame.size[0]*save_service->current_buffer->frame.size[1]*sizeof(unsigned char));
-	
-	for (i = 0; i < getDepth(info->bigbuffer_position, save_service->current_buffer->frame.color_coding,
-				 save_service->current_buffer->frame.size[1], save_service->current_buffer->frame.size[0]); i++) {
-	  convert_for_pvn(info->bigbuffer, save_service->current_buffer->frame.size[0],
-			  save_service->current_buffer->frame.size[1], i, save_service->current_buffer->frame.color_coding, tmp_buf);
-	  fwrite(tmp_buf, 3*save_service->current_buffer->frame.size[0]*save_service->current_buffer->frame.size[1], 1, fd);
-	}
-	free(tmp_buf);
-	tmp_buf=NULL;
-      }
-      break;
-    default:
-      fprintf(stderr,"Error: Ram buffer should be accessible only in video mode\n");
-      break;
-    }
-  }
-  //fprintf(stderr,"handling FD\n");
   
   if (fd!=NULL) {
     fclose(fd);
