@@ -28,7 +28,7 @@ V4lStartThread(camera_t* cam)
 
   stemp=(char*)malloc(STRING_SIZE*sizeof(char));
 
-  v4l_service=GetService(camera, SERVICE_V4L);
+  v4l_service=GetService(cam, SERVICE_V4L);
 
   if (v4l_service==NULL) { // if no V4L service running...
     v4l_service=(chain_t*)malloc(sizeof(chain_t));
@@ -48,23 +48,21 @@ V4lStartThread(camera_t* cam)
     /* setup v4l_thread: handles, ...*/
     pthread_mutex_lock(&v4l_service->mutex_data);
     
-    info->period=cam->prefs.v4l_period;
     CommonChainSetup(cam, v4l_service,SERVICE_V4L);
     
-    info->v4l_buffer=NULL;
-    //info->counter=0;
-
-    // COPY CAM->PREFS HERE
-    strcpy(info->v4l_dev_name,cam->prefs.v4l_dev_name);
+    info->frame.image=NULL;
+    info->frame.allocated_image_bytes=0;
 
     // open V4L device
     info->v4l_dev=-1;
-    info->v4l_dev = open(info->v4l_dev_name, O_RDWR);
+    info->v4l_dev = open(cam->prefs.v4l_dev_name, O_RDWR);
     if (info->v4l_dev < 0) {
-      sprintf(stemp,"Failed to open V4L device %s",info->v4l_dev_name);
-      MainError(stemp);
+      sprintf(stemp,"Failed to open V4L device %s",cam->prefs.v4l_dev_name);
+      Error(stemp);
       free(stemp);
+      stemp=NULL;
       FreeChain(v4l_service);
+      v4l_service=NULL;
       return(-1);
     }
 
@@ -79,9 +77,14 @@ V4lStartThread(camera_t* cam)
       RemoveChain(cam, v4l_service);
       pthread_mutex_unlock(&v4l_service->mutex_struct);
       pthread_mutex_unlock(&v4l_service->mutex_data);
-      free(info->v4l_buffer);
+      if ((info->frame.image!=NULL)&&(info->frame.allocated_image_bytes>0)) {
+	free(info->frame.image);
+	info->frame.allocated_image_bytes=0;
+      }
       free(stemp);
+      stemp=NULL;
       FreeChain(v4l_service);
+      v4l_service=NULL;
       return(-1);
     }
 
@@ -90,27 +93,11 @@ V4lStartThread(camera_t* cam)
     
   }
   free(stemp);
+  stemp=NULL;
   return (1);
 }
 
 
-void*
-V4lCleanupThread(void* arg) 
-{
-  chain_t* v4l_service;
-  v4lthread_info_t *info;
-
-  v4l_service=(chain_t*)arg;
-  info=(v4lthread_info_t*)v4l_service->data;
-  /* Specific cleanups: */
-
-  /* Mendatory cleanups: */
-  pthread_mutex_unlock(&v4l_service->mutex_data);
-
-  return(NULL);
-}
-
-
 void*
 V4lThread(void* arg)
 {
@@ -118,10 +105,13 @@ V4lThread(void* arg)
   v4lthread_info_t *info=NULL;
   long int skip_counter;
   float tmp;
+  struct video_picture p;
   
   v4l_service=(chain_t*)arg;
   pthread_mutex_lock(&v4l_service->mutex_data);
   info=(v4lthread_info_t*)v4l_service->data;
+  camera_t *cam=v4l_service->camera;
+
   skip_counter=0;
   v4l_service->processed_frames=0;
 
@@ -132,8 +122,10 @@ V4lThread(void* arg)
   pthread_mutex_unlock(&v4l_service->mutex_data);
   
   // time inits:
-  v4l_service->prev_time = times(&v4l_service->tms_buf);
-  v4l_service->fps_frames=0;
+  v4l_service->prev_time = 0;
+  v4l_service->prev_period = 0;
+  v4l_service->drop_warning = 0;
+  v4l_service->processed_frames=0;
 
   while (1) { 
     /* Clean cancel handlers */
@@ -144,40 +136,74 @@ V4lThread(void* arg)
     else {
       pthread_mutex_unlock(&info->mutex_cancel);
       pthread_mutex_lock(&v4l_service->mutex_data);
-      if(RollBuffers(v4l_service)) { // have buffers been rolled?
+      if(GetBufferFromPrevious(v4l_service)) { // have buffers been rolled?
 	// check params
 	V4lThreadCheckParams(v4l_service);
 	
-	convert_to_rgb(v4l_service->current_buffer, info->v4l_buffer);
+	/* IF we have mono data then set V4L for mono(grey) output */
+	/* Only do this ONCE before writing the first frame */
+	if (((v4l_service->current_buffer->frame.color_coding == DC1394_COLOR_CODING_MONO8) ||
+	     (v4l_service->current_buffer->frame.color_coding == DC1394_COLOR_CODING_RAW8)) && v4l_service->processed_frames==0) {
+	  Warning("Setting V4L device to GREY palette");
+	  if (ioctl(info->v4l_dev,VIDIOCGPICT,&p) < 0) 
+	    Error("ioctl(VIDIOCGPICT) error");
+	  else {
+	    p.palette = VIDEO_PALETTE_GREY;
+	    if (ioctl(info->v4l_dev,VIDIOCSPICT,&p) < 0) 
+	      Error("ioctl(VIDIOCSPICT) Error");
+	  }
+	}
 
-	swap_rb(info->v4l_buffer, v4l_service->current_buffer->width*v4l_service->current_buffer->height*3);
+	// Convert to RGB unless we are using direct GREY palette
+	if ((v4l_service->current_buffer->frame.color_coding != DC1394_COLOR_CODING_MONO8) &&
+	    (v4l_service->current_buffer->frame.color_coding != DC1394_COLOR_CODING_RAW8)) {
+	  convert_to_rgb(&v4l_service->current_buffer->frame, &info->frame);
+	  swap_rb(info->frame.image, v4l_service->current_buffer->frame.size[0]*v4l_service->current_buffer->frame.size[1]*3);
+	}
 
-	if (v4l_service->current_buffer->width!=-1) {
-	  if (skip_counter>=(info->period-1)) {
+	if (v4l_service->current_buffer->frame.size[0]!=-1) {
+	  if (skip_counter>=(cam->prefs.v4l_period-1)) {
 	    skip_counter=0;
-	    write(info->v4l_dev,info->v4l_buffer,v4l_service->current_buffer->width*v4l_service->current_buffer->height*3);
-	    v4l_service->fps_frames++;
+	    if ((v4l_service->current_buffer->frame.color_coding != DC1394_COLOR_CODING_MONO8) &&
+		(v4l_service->current_buffer->frame.color_coding != DC1394_COLOR_CODING_RAW8))
+	      write(info->v4l_dev,info->frame.image,v4l_service->current_buffer->frame.size[0]*v4l_service->current_buffer->frame.size[1]*3);
+	    else
+	      write(info->v4l_dev,v4l_service->current_buffer->frame.image,v4l_service->current_buffer->frame.size[0]*v4l_service->current_buffer->frame.size[1]);
 	    v4l_service->processed_frames++;
 	  }
 	  else
 	    skip_counter++;
 	  
-	  // FPS display
-	  v4l_service->current_time=times(&v4l_service->tms_buf);
-	  tmp=(float)(v4l_service->current_time-v4l_service->prev_time)/sysconf(_SC_CLK_TCK);
-	  if (tmp==0)
+	  // FPS computation:
+	  tmp=((float)(v4l_service->current_buffer->frame.timestamp-v4l_service->prev_time))/1000000.0;
+	  if (v4l_service->prev_time==0) {
 	    v4l_service->fps=fabs(0.0);
-	  else
-	    v4l_service->fps=fabs((float)v4l_service->fps_frames/tmp);
+	  }
+	  else {
+	    if (tmp==0)
+	      v4l_service->fps=fabs(0.0);
+	    else
+	      v4l_service->fps=fabs(1/tmp);
+	  }
+	  if (v4l_service->prev_time!=0) {
+	    v4l_service->prev_period=tmp;
+	  }
+	  // produce a drop warning if the period difference is over 50%
+	  if (v4l_service->prev_period!=0) {
+	    if (fabs(v4l_service->prev_period-tmp)/(v4l_service->prev_period/2+tmp/2)>=.5)
+	      v4l_service->drop_warning++;
+	  }
+	  v4l_service->prev_time=v4l_service->current_buffer->frame.timestamp;
 
 	}
+	PublishBufferForNext(v4l_service);
 	pthread_mutex_unlock(&v4l_service->mutex_data);
       }
       else {
 	pthread_mutex_unlock(&v4l_service->mutex_data);
       }
     }
-    usleep(0);
+    usleep(100);
   }
 
   pthread_mutex_unlock(&info->mutex_cancel);
@@ -208,16 +234,18 @@ V4lStopThread(camera_t* cam)
     RemoveChain(cam,v4l_service);
     
     /* Do custom cleanups here...*/
-    if (info->v4l_buffer!=NULL) {
-      free(info->v4l_buffer);
-      info->v4l_buffer=NULL;
+    if ((info->frame.image!=NULL)&&(info->frame.allocated_image_bytes>0)) {
+      free(info->frame.image);
+      info->frame.allocated_image_bytes=0;
     }
+
     close(info->v4l_dev);
 
     /* Mendatory cleanups: */
     pthread_mutex_unlock(&v4l_service->mutex_struct);
     pthread_mutex_unlock(&v4l_service->mutex_data);
     FreeChain(v4l_service);
+    v4l_service=NULL;
     
   }
   
@@ -229,85 +257,39 @@ V4lThreadCheckParams(chain_t *v4l_service)
 {
 
   v4lthread_info_t *info;
-  int buffer_size_change=0;
   info=(v4lthread_info_t*)v4l_service->data;
 
-  // copy harmless parameters anyway:
-  v4l_service->local_param_copy.bpp=v4l_service->current_buffer->bpp;
-  v4l_service->local_param_copy.bayer_pattern=v4l_service->current_buffer->bayer_pattern;
-
   // if some parameters changed, we need to re-allocate the local buffers and restart the v4l
-  if ((v4l_service->current_buffer->width!=v4l_service->local_param_copy.width)||
-      (v4l_service->current_buffer->height!=v4l_service->local_param_copy.height)||
-      (v4l_service->current_buffer->bytes_per_frame!=v4l_service->local_param_copy.bytes_per_frame)||
-      (v4l_service->current_buffer->mode!=v4l_service->local_param_copy.mode)||
-      (v4l_service->current_buffer->format!=v4l_service->local_param_copy.format)||
-      // check F7 color mode change
-      ((v4l_service->current_buffer->format==FORMAT_SCALABLE_IMAGE_SIZE)&&
-       (v4l_service->current_buffer->format7_color_mode!=v4l_service->local_param_copy.format7_color_mode)
-       ) ||
-      // check bayer and stereo decoding
-      (v4l_service->current_buffer->stereo_decoding!=v4l_service->local_param_copy.stereo_decoding)||
-      (v4l_service->current_buffer->bayer!=v4l_service->local_param_copy.bayer)
-      ) {
-    if (v4l_service->current_buffer->width*v4l_service->current_buffer->height!=
-	v4l_service->local_param_copy.width*v4l_service->local_param_copy.height) {
-      buffer_size_change=1;
-    }
-    else {
-      buffer_size_change=0;
-    }
-    
-    // copy all new parameters:
-    v4l_service->local_param_copy.width=v4l_service->current_buffer->width;
-    v4l_service->local_param_copy.height=v4l_service->current_buffer->height;
-    v4l_service->local_param_copy.bytes_per_frame=v4l_service->current_buffer->bytes_per_frame;
-    v4l_service->local_param_copy.mode=v4l_service->current_buffer->mode;
-    v4l_service->local_param_copy.format=v4l_service->current_buffer->format;
-    v4l_service->local_param_copy.format7_color_mode=v4l_service->current_buffer->format7_color_mode;
-    v4l_service->local_param_copy.stereo_decoding=v4l_service->current_buffer->stereo_decoding;
-    v4l_service->local_param_copy.bayer=v4l_service->current_buffer->bayer;
-    v4l_service->local_param_copy.buffer_image_bytes=v4l_service->current_buffer->buffer_image_bytes;
-    
-    // DO SOMETHING
-    if (buffer_size_change!=0) {
-      
-      // clear buffer
-      if (info->v4l_buffer!=NULL) {
-	free(info->v4l_buffer);
-	info->v4l_buffer=NULL;
-      }
+  if ((v4l_service->current_buffer->frame.size[0]!=v4l_service->local_param_copy.frame.size[0])||
+      (v4l_service->current_buffer->frame.size[1]!=v4l_service->local_param_copy.frame.size[1]) ) {
 
-      // create buffer
-      info->v4l_buffer=(unsigned char*)malloc(v4l_service->current_buffer->width*v4l_service->current_buffer->height
-					      *3*sizeof(unsigned char));
-      if (info->v4l_buffer==NULL) {
-	fprintf(stderr,"Can't allocate buffer! Aiiieee!\n");
-      }
-      
-      // STOPING THE PIPE MIGHT BE NECESSARY HERE
-
-      // "start pipe"      
-      if (ioctl (info->v4l_dev, VIDIOCGCAP, &info->vid_caps) == -1) {
-	perror ("ioctl (VIDIOCGCAP)");
-      }
-      if (ioctl (info->v4l_dev, VIDIOCGPICT, &info->vid_pic)== -1) {
-	perror ("ioctl VIDIOCGPICT");
-      }
-      info->vid_pic.palette = VIDEO_PALETTE_RGB24;
-      if (ioctl (info->v4l_dev, VIDIOCSPICT, &info->vid_pic)== -1) {
-	perror ("ioctl VIDIOCSPICT");
-      }
-      if (ioctl (info->v4l_dev, VIDIOCGWIN, &info->vid_win)== -1) {
-	perror ("ioctl VIDIOCGWIN");
-      }
-      info->vid_win.width=v4l_service->current_buffer->width;
-      info->vid_win.height=v4l_service->current_buffer->height;
-      if (ioctl (info->v4l_dev, VIDIOCSWIN, &info->vid_win)== -1) {
-	perror ("ioctl VIDIOCSWIN");
-      }
+    // STOPING THE PIPE MIGHT BE NECESSARY HERE
+    
+    // "start pipe"      
+    if (ioctl (info->v4l_dev, VIDIOCGCAP, &info->vid_caps) == -1) {
+      perror ("ioctl (VIDIOCGCAP)");
+    }
+    if (ioctl (info->v4l_dev, VIDIOCGPICT, &info->vid_pic)== -1) {
+      perror ("ioctl VIDIOCGPICT");
+    }
+    info->vid_pic.palette = VIDEO_PALETTE_RGB24;
+    if (ioctl (info->v4l_dev, VIDIOCSPICT, &info->vid_pic)== -1) {
+      perror ("ioctl VIDIOCSPICT");
+    }
+    if (ioctl (info->v4l_dev, VIDIOCGWIN, &info->vid_win)== -1) {
+      perror ("ioctl VIDIOCGWIN");
+    }
+    info->vid_win.width=v4l_service->current_buffer->frame.size[0];
+    info->vid_win.height=v4l_service->current_buffer->frame.size[1];
+    if (ioctl (info->v4l_dev, VIDIOCSWIN, &info->vid_win)== -1) {
+      perror ("ioctl VIDIOCSWIN");
     }
   }
+
+  // copy all new parameters:
+  memcpy(&v4l_service->local_param_copy, v4l_service->current_buffer,sizeof(buffer_t));
+  v4l_service->local_param_copy.frame.allocated_image_bytes=0; // to avoid bad free...
+  
 }
 
 void
