@@ -37,11 +37,8 @@ typedef struct _xvinfo
 void
 GetFormat7Capabilities(camera_t* cam);
 
-void
-GetFormat7ModeInfo(camera_t* cam, int mode_id);
-
-void
-SwitchToNearestFPS(quadlet_t compat, int current);
+unsigned int
+SwitchToNearestFPS(dc1394framerates_t *framerates, dc1394framerate_t current);
 
 void
 ChangeModeAndFormat         (GtkMenuItem     *menuitem,
@@ -56,20 +53,12 @@ void
 GetContextStatus(void);
 
 void
-GrabSelfIds(raw1394handle_t* handles, int portmax);
-
+GrabSelfIds(camera_t *cams);
+//GrabSelfIds(raw1394handle_t* handles, int portmax);
+/*
 void
-SetChannels(void);
-
-void
-MainError(const char *string);
-
-void
-MainStatus(const char *string);
-
-void
-MessageBox(gchar *message);
-
+SetIsoChannels(void);
+*/
 void
 SetScaleSensitivity(GtkWidget* widget, int feature, dc1394bool_t sense);
 
@@ -94,11 +83,19 @@ SetFormat7Crop(int sx, int sy, int px, int py, int mode);
 int
 NearestValue(int value, int step, int min, int max);
 
+void
+IsOptionAvailableWithFormat(camera_t* cam,int* bayer, int* stereo, int *bpp16);
+
+/*
 void
 window_set_icon(GtkWidget* window);
+*/
 
 void
 GetXvInfo(xvinfo_t *xvinfo);
 
+void
+AutoIsoReceive(dc1394switch_t pwr);
+
 #endif // __TOOLS_H__
 
