@@ -63,17 +63,13 @@ UpdatePrefsSaveFrame(void)
 
   // thread presence blanking: default some to ON
   gtk_widget_set_sensitive(lookup_widget(main_window,"prefs_save_filename_frame"), TRUE);
-  gtk_widget_set_sensitive(lookup_widget(main_window,"ram_buffer_frame"), TRUE);
 
   // normal:
   gtk_widget_set_sensitive(lookup_widget(main_window,"save_to_dir"), (!is_video)&&(!camera->prefs.save_to_stdout));
-  gtk_widget_set_sensitive(lookup_widget(main_window,"ram_buffer_frame"), is_video);
-  gtk_widget_set_sensitive(lookup_widget(main_window,"use_ram_buffer"), is_video);
 
   // thread presence blanking:
   if (GetService(camera,SERVICE_SAVE)!=NULL) {
     gtk_widget_set_sensitive(lookup_widget(main_window,"prefs_save_filename_frame"), FALSE);
-    gtk_widget_set_sensitive(lookup_widget(main_window,"ram_buffer_frame"), FALSE);
   }
   UpdateSaveFilenameFrame();
 
@@ -849,6 +845,7 @@ UpdateSaveFilenameFrame(void)
 #endif
   case SAVE_FORMAT_PPM:
   case SAVE_FORMAT_RAW:
+  case SAVE_FORMAT_TIFF:
     // first handle the case of save-to-dir
     if (camera->prefs.save_to_dir==0) {
       switch (camera->prefs.save_append) {
