@@ -56,13 +56,14 @@ WatchThread(void *arg)
 {
   watchthread_info_t *info;
   info=(watchthread_info_t*)arg;
- 
+  dc1394video_mode_t video_mode;
   
   pthread_mutex_lock(&info->mutex_area);
   pthread_setcancelstate(PTHREAD_CANCEL_DISABLE,NULL);
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED,NULL);
   pthread_mutex_unlock(&info->mutex_area);
 
+  dc1394_video_get_mode(camera->camera_info, &video_mode);
   while (1) {
     pthread_mutex_lock(&info->mutex_cancel_watch);
     if (info->cancel_watch_req>0) {
@@ -74,25 +75,27 @@ WatchThread(void *arg)
       pthread_mutex_lock(&info->mutex_area);
       
       if (info->crop>0) {
-	if (camera->misc_info.format==FORMAT_SCALABLE_IMAGE_SIZE) {
-	  if (dc1394_query_format7_image_position(camera->camera_info.handle, camera->camera_info.id, camera->misc_info.mode,
-						  &camera->format7_info.mode[camera->misc_info.mode-MODE_FORMAT7_MIN].pos_x,
-						  &camera->format7_info.mode[camera->misc_info.mode-MODE_FORMAT7_MIN].pos_y)!=DC1394_SUCCESS)
-	    MainError("Could not get format7 image position");
+	if ((video_mode >= DC1394_VIDEO_MODE_FORMAT7_MIN) &&
+	    (video_mode <= DC1394_VIDEO_MODE_FORMAT7_MAX)) {
+	  if (dc1394_format7_get_image_position(camera->camera_info, video_mode,
+						  &camera->format7_info.modeset.mode[video_mode-DC1394_VIDEO_MODE_FORMAT7_MIN].pos_x,
+						  &camera->format7_info.modeset.mode[video_mode-DC1394_VIDEO_MODE_FORMAT7_MIN].pos_y)!=DC1394_SUCCESS)
+	    Error("Could not get format7 image position");
 	  // if we did reset to max size, don't do the addition:
-	  if ((info->size[0]==camera->format7_info.mode[camera->misc_info.mode-MODE_FORMAT7_MIN].max_size_x)&&
-	      (info->size[1]==camera->format7_info.mode[camera->misc_info.mode-MODE_FORMAT7_MIN].max_size_y))
-	    SetFormat7Crop(info->size[0],info->size[1],info->pos[0],info->pos[1],camera->misc_info.mode);
+	  if ((info->size[0]==camera->format7_info.modeset.mode[video_mode-DC1394_VIDEO_MODE_FORMAT7_MIN].max_size_x)&&
+	      (info->size[1]==camera->format7_info.modeset.mode[video_mode-DC1394_VIDEO_MODE_FORMAT7_MIN].max_size_y))
+	    SetFormat7Crop(info->size[0],info->size[1],info->pos[0],info->pos[1],video_mode);
 	  else {
 	    SetFormat7Crop(info->size[0],info->size[1],
-			   info->pos[0]+camera->format7_info.mode[camera->misc_info.mode-MODE_FORMAT7_MIN].pos_x,
-			   info->pos[1]+camera->format7_info.mode[camera->misc_info.mode-MODE_FORMAT7_MIN].pos_y,
-			   camera->misc_info.mode);
+			   info->pos[0]+camera->format7_info.modeset.mode[video_mode-DC1394_VIDEO_MODE_FORMAT7_MIN].pos_x,
+			   info->pos[1]+camera->format7_info.modeset.mode[video_mode-DC1394_VIDEO_MODE_FORMAT7_MIN].pos_y,
+			   video_mode);
 	  }
-	  UpdateFormat7BppRange();
+	  //UpdateFormat7Ranges();
+	  // no need to rebuild at all: setformat7crop includes rebuilding 
 	}
 	else {
-	  SetFormat7Crop(info->size[0],info->size[1],info->pos[0],info->pos[1],camera->misc_info.mode);
+	  SetFormat7Crop(info->size[0],info->size[1],info->pos[0],info->pos[1],video_mode);
 	}
 	info->crop=0;
       }
@@ -124,11 +127,14 @@ WatchStopThread(watchthread_info_t* info)
 void
 GetValidF7Crop(watchthread_info_t *wtinfo, chain_t* display_service) {
 
-  Format7ModeInfo_t* f7info;
+  dc1394format7mode_t* f7info;
+  dc1394video_mode_t video_mode;
   int mov[2];
+  dc1394_video_get_mode(camera->camera_info, &video_mode);
   
-  if (camera->misc_info.format==FORMAT_SCALABLE_IMAGE_SIZE) {
-    f7info=&camera->format7_info.mode[camera->misc_info.mode-MODE_FORMAT7_MIN];
+  if ((video_mode >= DC1394_VIDEO_MODE_FORMAT7_MIN) &&
+      (video_mode <= DC1394_VIDEO_MODE_FORMAT7_MAX)) {
+    f7info=&camera->format7_info.modeset.mode[video_mode-DC1394_VIDEO_MODE_FORMAT7_MIN];
 
     // step_pos=step if no step_pos is supported.
     wtinfo->pos[0]=wtinfo->upper_left[0]-wtinfo->upper_left[0]%f7info->unit_pos_x;
@@ -170,10 +176,10 @@ GetValidF7Crop(watchthread_info_t *wtinfo, chain_t* display_service) {
       wtinfo->pos[0]=0;
     if (wtinfo->pos[1]<0)
       wtinfo->pos[1]=0;
-    if (wtinfo->pos[0]+wtinfo->size[0]-1>display_service->current_buffer->width)
-      wtinfo->pos[0]-=wtinfo->pos[0]+wtinfo->size[0]-display_service->current_buffer->width; // there was a +1 here
-    if (wtinfo->pos[1]+wtinfo->size[1]-1>display_service->current_buffer->height)
-      wtinfo->pos[1]-=wtinfo->pos[1]+wtinfo->size[1]-display_service->current_buffer->height; // there was a +1 here
+    if (wtinfo->pos[0]+wtinfo->size[0]-1>display_service->current_buffer->frame.size[0])
+      wtinfo->pos[0]-=wtinfo->pos[0]+wtinfo->size[0]-display_service->current_buffer->frame.size[0]; // there was a +1 here
+    if (wtinfo->pos[1]+wtinfo->size[1]-1>display_service->current_buffer->frame.size[1])
+      wtinfo->pos[1]-=wtinfo->pos[1]+wtinfo->size[1]-display_service->current_buffer->frame.size[1]; // there was a +1 here
 
   }
   else {
